/* AVR Simon [avrsimon]
 *
 * avrsimon.h
 *
 * Created by Joe Grand, Grand Idea Studio, Inc.
 * http://www.grandideastudio.com
 *
 * This file contains code for the Atmel ATTiny2313V
 *  
 * Distributed under a Creative Commons Attribution-Share Alike 3.0 Unported license:
 * http://creativecommons.org/licenses/by-sa/3.0/
 *
 * Change log:
 *
 * Version 1.0, Initial release, October 26, 2009
 
 * Version 1.1, September 7, 2010
 * Changes (primarily to allow code to compile under avr-gcc version 4):
 * - Forced no-inlining of delay_ms() to save code space
 * 
 */

#ifndef __INC_AVRSIMON_H__
#define __INC_AVRSIMON_H__

// macros to set/clear a bit in a register
#define _cbi(port,bit) (port) &= ~(1 << (bit))	// clear bit
#define _sbi(port,bit) (port) |= (1 << (bit))	// set bit
#define _ledoff(bit)   (_sbi(LED_PORT, bit))    // turn led off 
#define _ledon(bit)	   (_cbi(LED_PORT, bit))    // turn led on 


// port definitions
// LEDs
#define LED_PORT		PORTD
#define LED_PIN			PIND
#define LED_DDR			DDRD
#define LED_RED         PD0
#define LED_GREEN       PD1
#define LED_YELLOW      PD2
#define LED_BLUE        PD3

// pushbutton switches
#define SW_PORT			PORTB
#define SW_PIN			PINB
#define SW_DDR			DDRB
#define SW_GREEN        PB0	
#define SW_RED          PB1
#define SW_BLUE         PB4
#define SW_YELLOW       PB3

#define SW_NO_BUTTON	0x1B
#define SW_GREEN_MASK   0x01
#define SW_RED_MASK     0x02
#define SW_YELLOW_MASK  0x08
#define SW_BLUE_MASK    0x10

// piezo buzzer
#define TONE_PIN        PB2


// constants
#define TIMEOUT_DELAY_EASY			 5      // delay (in s) to wait for a player to make a move choice, easy mode
#define TIMEOUT_DELAY_HARD			 2		// delay (in s) to wait for a player to make a move choice, hard mode
#define BETWEEN_MOVE_ON_DELAY		 300    // delay (in ms) to show the current moves
#define BETWEEN_MOVE_OFF_DELAY_EASY  300    // delay (in ms) to pause in between showing the moves, easy mode
#define BETWEEN_MOVE_OFF_DELAY_HARD  100	// delay (in ms) to pause in between showing the moves, hard mode

#define EEPROM_SIZE		128		// size of ATtiny2313's internal EEPROM used for gameplay storage
#define RED_MOVE        0x00    // move values stored in internal EEPROM
#define GREEN_MOVE      0x01
#define BLUE_MOVE       0x02
#define YELLOW_MOVE     0x03

#define SILENT_MODE     0x01    // game configuration flags (set on power-up), any combination of these modes can be entered
#define BLIND_MODE      0x02
#define EXPERT_MODE     0x04
#define REVERSE_MODE    0x08

// max counts for notes (8MHz clock)
#define C_note		119
#define Cs_note		113
#define D_note		106
#define Ds_note		100
#define E_note		95
#define F_note		89
#define Fs_note		84
#define G_note		80
#define Gs_note		75
#define A_note		71
#define As_note		67
#define B_note		63

// structs
struct note_struct	// struct for storing music 
{
	unsigned char octave;
	unsigned char note;
	unsigned int length;
};


// function prototypes
void simon_config(void);
int simon_game(void);
void simon_play_moves(void);
int simon_read_moves(void);
void red_move(void);
void green_move(void);
void yellow_move(void);
void blue_move(void);
void simon_failed_input(void);
void play_note(unsigned char octave, unsigned char note, unsigned int length);
void hardware_init(void);
unsigned char rand(void);
void delay_ms(unsigned int ms) __attribute__((noinline));
void all_led_off(void);

#endif
